#
# Copyright 2010-2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Greengrass lambda function to perform Image Classification with example model
# Resnet-50 that was compiled by DLC.
#
#
import logging
import random
import os

import cv2
import numpy as np
from dlr import DLRModel

import greengrasssdk
import utils
import camera

# Create MQTT client
mqtt_client = greengrasssdk.client('iot-data')

# Create camera
my_camera = camera.Camera()

# Initialize logger
customer_logger = logging.getLogger(__name__)

# Initialize example Resnet-50 model
model_resource_path = os.environ.get('MODEL_PATH', '/ml_model')
input_shape = {'data': [1, 3, 224, 224]}
output_shape = [1, 1000]
dlr_model = DLRModel(model_resource_path, input_shape, output_shape, 'gpu')

# Read synset file
synset_path = os.path.join(model_resource_path, 'synset.txt')
with open(synset_path, 'r') as f:
    synset = eval(f.read())


def predict(image_data):
    r"""
    Predict image with DLR. The result will be published
    to MQTT topic '/resnet-50/predictions'.

    :param image: numpy array of the Image inference with.
    """
    image_data = utils.transform_image(image_data)

    # Run DLR to perform inference with DLC optimized model
    model_output = dlr_model.run(image_data)

    # Prepare result
    probabilities = model_output[0]
    max_score_id = np.argmax(probabilities)
    max_score = probabilities[max_score_id]
    predicted_class = synset[max_score_id]
    result = 'Inference Result: "{}" with probability {}.'\
        .format(predicted_class, max_score)

    # Send result
    send_mqtt_message(
        'Prediction Result: {}'.format(result))


def predict_from_image():
    r"""
    Look for raw image files in your lambda directory
    and perform inference.
    """
    photo_album = []
    for root, dirs, files in os.walk("./", topdown=False):
        image_files = filter(lambda f: (f.endswith('.jpg') or f.endswith('.jpeg') or f.endswith('.png')), files)
        photo_album.extend(map((lambda f: os.path.join(root, f)), image_files))

    if len(photo_album) == 0:
        send_mqtt_message(
            'No image file found. This model is not able to predict empty image, yet...')

    else:
        image = random.choice(photo_album)
        send_mqtt_message(
            'Perform prediction on {}...'.format(image))

        image_data = cv2.imread(image)
        image_data = cv2.resize(image_data, (224, 224))
        predict(image_data)


def predict_from_cam():
    r"""
    Predict with the photo taken from onboard camera
    on your Jetson TX2.
    """
    send_mqtt_message("Taking a photo...")
    image_data = my_camera.capture_image(224, 224)

    send_mqtt_message("Start predicting...")
    predict(image_data)


def send_mqtt_message(message):
    r"""
    Publish message to the MQTT topic:
    '/resnet-50/predictions'.

    :param message: message to publish
    """
    mqtt_client.publish(topic='/resnet-50/predictions',
                        payload=message)


# The lambda to be invoked in Greengrass
def handler(event, context):
    try:
        # You can uncomment below code to use camera.
        # Please check README for requirements.
        #
        # predict_from_cam()

        predict_from_image()

    except Exception as e:
        customer_logger.exception(e)
        send_mqtt_message(
            'Exception occurred during prediction. Please check logs for troubleshooting: /greengrass/ggc/var/log.')
